-- Name: Valley Escort
-- Author: Delta99
-- Date Created: 23 Jan 2016
--
-- Based on my Valley Patrol mission and expanded.

-- Define some debug variables

D99_DebugRedSpawnPlanesDisable = D99_DebugRedSpawnPlanesDisable or false
D99_DebugRedSpawnGroundDisable = D99_DebugRedSpawnGroundDisable or false
D99_DebugTrace = D99_DebugTrace or false

function D99DebugTrace(output)
  if D99_DebugTrace then
    BASE:E(output)
  end
end 

-- Global Menu Arrays:

MenuBlueValley = {}
MenuBlueValleyProceed = {}
MenuBlueValleyBackToPatrol = {}
MenuBlueValleyRTB = {}
MenuBlueValleyStatus = {}
MenuBlueValleyCallCAS = {}
MenuBlueValleyStatusAll = {}
 
-- Define the SET of CLIENTs from the red coalition for each of the airports.
-- This SET is filled during startup only
local RU_PlanesClientSetVody = SET_CLIENT:New():FilterPrefixes("RU Client Vody")
local RU_PlanesClientSetMozdok = SET_CLIENT:New():FilterPrefixes("RU Client Mozdok")

if (not D99_DebugRedSpawnPlanesDisable) then
  -- We define an array of potential spawn planes (using templates from the editor).
  -- The InitCleanup will attempt to cleanup planes stuck at the airbase.
  -- The InitRandomizeTemplate picks a random plane from the RU_PlanesList to spawn using
  -- "AI_RU" template from the editor.
  local RU_PlanesList = {'AI RU 1', 'AI RU 2', 'AI RU 3'}
  local RU_PlanesSpawnVody = SPAWN:New( "AI RU Vody" ):InitLimit(4, 0):InitRandomizeTemplate(RU_PlanesList):InitCleanUp( 30 )
  local RU_PlanesSpawnMozdok = SPAWN:New( "AI RU Mozdok" ):InitLimit(4, 0):InitRandomizeTemplate(RU_PlanesList):InitCleanUp( 30 )
  
  -- Start the AI_BALANCER, using the SET of red CLIENTs, and the SPAWN object as a parameter.
  -- Try setting up an AI_BALANCER for Mineralnye Vody and Mozdok.
  -- Using the AI_BALANCER allows the AI to take these planes if there are no human players for their spots.
  -- InitSpawnInterval sets the interval between min and max seconds to spawn so that they are not all
  -- spawned right away.
  RU_AI_BalancerVody = AI_BALANCER:New( RU_PlanesClientSetVody, RU_PlanesSpawnVody ):InitSpawnInterval(90, 300)
  RU_AI_BalancerMozdok = AI_BALANCER:New( RU_PlanesClientSetMozdok, RU_PlanesSpawnMozdok ):InitSpawnInterval(90, 300)
  
  -- Create the first polygon zone ...
  local PatrolZoneGroup1 = GROUP:FindByName( "PatrolZone1" )
  local PatrolZone1 = ZONE_POLYGON:New( "PatrolZone1", PatrolZoneGroup1 )
  
  -- Create the second polygon zone ...
  local PatrolZoneGroup2 = GROUP:FindByName( "PatrolZone2" )
  local PatrolZone2 = ZONE_POLYGON:New( "PatrolZone2", PatrolZoneGroup2 )
  
  -- Now, create an array of these zones ...
  local PatrolZoneArray = { PatrolZone1, PatrolZone2 }
  
  local EngageZoneAll = GROUP:FindByName( "EngageZoneAll" )
  local CapEngageZone = ZONE_POLYGON:New( "EngageZoneAll", EngageZoneAll )

  -- TODO: Have these AI_BALANCER functions call 1 common routine  
  function RU_AI_BalancerVody:OnAfterSpawned( SetGroup, From, Event, To, AIGroup )
    -- For now set the Patrol Zone and Engagement Zone to the same. Was having issues when I
    -- used the patrol zones from above as the engagement zones as well so instead creating
    -- a new one here that encompasses both.
  
    local ranPatrolZone = PatrolZoneArray[math.random( 1, 2 )]
    local PatrolCAPZone = AI_CAP_ZONE:New(ranPatrolZone, 100, 4000, 600, 1200)
    PatrolCAPZone:ManageFuel(0.25, 45)
    PatrolCAPZone:SetControllable(AIGroup)
    
    -- Try using a different engagement zone than the patrol zone
    PatrolCAPZone:SetEngageZone(CapEngageZone)
    --PatrolCAPZone:SetEngageZone(ranPatrolZone)
    PatrolCAPZone:__Start(1)
  end

  function RU_AI_BalancerMozdok:OnAfterSpawned( SetGroup, From, Event, To, AIGroup )
    -- For now set the Patrol Zone and Engagement Zone to the same. Was having issues when I
    -- used the patrol zones from above as the engagement zones as well so instead creating
    -- a new one here that encompasses both.
  
    local ranPatrolZone = PatrolZoneArray[math.random( 1, 2 )]
    local PatrolCAPZone = AI_CAP_ZONE:New(ranPatrolZone, 100, 4000, 600, 1200)
    PatrolCAPZone:ManageFuel(0.25, 45)
    PatrolCAPZone:SetControllable(AIGroup)
    
    -- Try using a different engagement zone than the patrol zone
    PatrolCAPZone:SetEngageZone(CapEngageZone)
    --PatrolCAPZone:SetEngageZone(ranPatrolZone)
    PatrolCAPZone:__Start(1)
  end
end

if (not D99_DebugRedSpawnGroundDisable) then
  -- Lets spawn some defences at the RED airports
  
  RU_GroundVody = SPAWN:New("RU ZSU-23 Template 2"):InitRandomizePosition(true, 3800, 2400):Spawn()
  RU_GroundMozdok = SPAWN:New("RU ZSU-23 Template 3"):InitRandomizePosition(true, 3800, 2400):Spawn()

  -- Also InitRandomizeTemplate together with InitRandomizeZones not working well so going to try something different.
  
  -- Define the valley zones for the Russian ground units
  local numRedValleys = 7
  local ZoneRedValleyTemplate = {}
  local ZoneRedValley = {}
  
  for i=1, numRedValleys do
    ZoneRedValleyTemplate[i] = GROUP:FindByName("ZoneRedValley" .. i)
    ZoneRedValley[i] = ZONE_POLYGON:New("ZoneRedValley" .. i, ZoneRedValleyTemplate[i])
  end
  
  local RU_GroundArray = {"RU AAA ZU-23 Template 1", "RU AAA ZU-23 Template 2", "RU AAA ZU-23 Template 3", "RU ZSU-23 Template 1"}
  
  local numGroundGroups = 11   -- The number of ground groups to spawn across all the Valley zones
  RU_GroundGroups = {}
  
  for i=1,numGroundGroups do
    local tempRandGroup = math.random(1,table.getn(RU_GroundArray))
    local tempGrpAlias = "RUGroundGroup-" .. i
    RU_GroundGroups[i] = SPAWN:NewWithAlias(RU_GroundArray[tempRandGroup], tempGrpAlias):InitRandomizeZones(ZoneRedValley)
    RU_GroundGroups[i]:Spawn()
  end

  -- FIXME: I was trying to use InitLimit on this New() call but couldn't get anything to spawn at all.
  -- It could have something to do with it counting units rather than groups per say. I may come
  -- back to this and try it again. For now just calling spawn over and over works.
  
  -- FIXME: InitRandomize template is now working (wasn't for a while) but we have the above in its place as a workaround. Works nicely.
  --local RU_GroundSpawn = SPAWN:New("RU AAA ZU-23 Template"):InitRandomizeTemplate(RU_GroundArray):InitRandomizeZones({ZoneValley1, ZoneValley1, ZoneValley1})
  --local RU_Group1 = RU_GroundSpawn:Spawn()
  --local RU_Group2 = RU_GroundSpawn:Spawn()
  --local RU_Group3 = RU_GroundSpawn:Spawn()
end

-- Spawn the BLUE air defense groups at the airbases

US_GroundGudauta = SPAWN:New("US Air Defense Template 1"):InitRandomizePosition(true, 2800, 1600):Spawn()
US_GroundSakhumi = SPAWN:New("US Air Defense Template 2"):InitRandomizePosition(true, 2800, 1850):Spawn()
US_GroundSenaki = SPAWN:New("US Air Defense Template 3"):InitRandomizePosition(true, 2800, 1400):Spawn()
US_GroundKutaisi = SPAWN:New("US Air Defense Template 4"):InitRandomizePosition(true, 2800, 1400):Spawn()

-- Setup the AWACS Patrol and Escort flight
function spawnUSAWACS()
  USAWACSPatrolGroup = SPAWN:New("US AI AWACS 1"):Spawn()
  local tempRoute = GROUP:FindByName("US AI AWACS Route 1"):CopyRoute(0, 0, true, 100)
  USAWACSPatrolGroup:Route(tempRoute)
end

function spawnUSAWACSEscort()
  USAWACSEscortGroup = SPAWN:New("US AI AWACS Escort 1"):Spawn()
  local escortVec3 = POINT_VEC3:New(-500, 0, 200)
  local escortTask = USAWACSEscortGroup:TaskEscort(USAWACSPatrolGroup, escortVec3, nil, 46250, {"Air"})
  USAWACSEscortGroup:PushTask(escortTask, 5)
end

function USAWACSRTB()
  if (USAWACSPatrolGroup:IsAlive()) then
    local tempRoute = GROUP:FindByName("US AI AWACS Route 1"):CopyRoute(4, 0, false, 100)
    USAWACSPatrolGroup:Route(tempRoute)
  end
end

function USAWACSEscortRTB()
  if (USAWACSEscortGroup:IsAlive()) then
    local tempRoute = GROUP:FindByName("US AI AWACS Route 1"):CopyRoute(4, 0, false, 100)
    USAWACSEscortGroup:Route(tempRoute)
  end
end

spawnUSAWACS()
spawnUSAWACSEscort()

-- Going to test spawning Valley SEAD planes using similar method to AWACs rather than AI_CAS_ZONE.
-- I like how this is working better than AI_CAS_ZONE so going with this method

local numValleys = 7
local ZoneUSPatrolValleyTemplate = {}
local ZoneUSPatrolValley = {}
local ZoneUSEngageValleyTemplate = {}
local ZoneUSEngageValley = {}
local USValleyRoute = {}
local USValleyRouteEngage = {}
local USValleyRoutePatrol = {}
local USValleyRouteRTB = {}

for i=1, numValleys do
  ZoneUSEngageValleyTemplate[i] = GROUP:FindByName("USValleyEngageZone" .. i)
  ZoneUSEngageValley[i] = ZONE_POLYGON:New("USEngageValley" .. i, ZoneUSEngageValleyTemplate[i])

  ZoneUSPatrolValleyTemplate[i] = GROUP:FindByName("USValleyPatrolZone" .. i)
  ZoneUSPatrolValley[i] = ZONE_POLYGON:New("USPatrolValley" .. i, ZoneUSPatrolValleyTemplate[i])
  
  USValleyRoute[i] = GROUP:FindByName("US Route Valley " .. i):CopyRoute(0, 0, false, 100)
  USValleyRoutePatrol[i] = GROUP:FindByName("US Route Valley " .. i):CopyRoute(2, 0, false, 100)
  USValleyRouteEngage[i] = GROUP:FindByName("US Route Valley " .. i):CopyRoute(4, 0, false, 100)
  USValleyRouteRTB[i] = GROUP:FindByName("US Route Valley " .. i):CopyRoute(16, 0, false, 100)
end

USValleySEAD = {}
USValleySEADState = {}

function USValleySEADSpawned(SpawnGroup, valleyNum)
  D99DebugTrace("D99 in USValleySEADSpawned")
  D99DebugTrace(SpawnGroup)
  SpawnGroup:Route(USValleyRoute[valleyNum])

  tempGroupUnits = SpawnGroup:GetUnits()
  D99DebugTrace(tempGroupUnits)
  for key, tempUnit in pairs(tempGroupUnits) do
    tempUnitCallSign = tempUnit:GetCallsign()
    D99DebugTrace(tempUnitCallSign)
    USValleySEADState[tempUnitCallSign] = "Readying For Takeoff"
  end
end

function setUnitStateByName(unitName, stateString)
  local unit = UNIT:FindByName(unitName)
  local unitCallSign = unit:GetCallsign()
  USValleySEADState[unitCallSign] = stateString
end

function setUnitAndGroupState(group, stateString)
  D99DebugTrace(group)
  D99DebugTrace(stateString)
  group:SetState(group, "US_AI_CAS_State", stateString)
  local groupUnits = group:GetUnits()
  D99DebugTrace(groupUnits)
  for key, tempUnit in pairs(groupUnits) do
    D99DebugTrace(tempUnit)
    tempUnitCallSign = tempUnit:GetCallsign()
    D99DebugTrace(tempUnitCallSign)
    USValleySEADState[tempUnitCallSign] = stateString
  end
end

function spawnUSValleySEAD(valleyNum)
  -- TODO: Change the AWACS and Escort to be similar to this with the OnSpawnGroup stuff.
  USValleySEAD[valleyNum] = SPAWN:New("US AI CAS " .. valleyNum):OnSpawnGroup(USValleySEADSpawned, valleyNum):InitRepeatOnEngineShutDown():Spawn()

  USValleySEAD[valleyNum]:HandleEvent(EVENTS.Birth, 
    function (group, EventData)
      D99DebugTrace("D99 in Events.Birth")
      D99DebugTrace(EventData)
      USValleySEAD[valleyNum]:SetState(USValleySEAD[valleyNum], "US_AI_CAS_State", "Readying For Takeoff")

      setUnitStateByName(EventData.IniUnitName, "Readying For Takeoff")
    end
  )
  
  USValleySEAD[valleyNum]:HandleEvent(EVENTS.Takeoff, 
    function (group, EventData)
      D99DebugTrace("D99 in Events.Takeoff")
      D99DebugTrace(EventData)
      USValleySEAD[valleyNum]:SetState(USValleySEAD[valleyNum], "US_AI_CAS_State", "Enroute Patrol")
      setUnitStateByName(EventData.IniUnitName, "Enroute Patrol")
    end
  )  

  USValleySEAD[valleyNum]:HandleEvent(EVENTS.Land, 
    function (group, EventData)
      D99DebugTrace("D99 in Events.Land")
      D99DebugTrace(EventData)
      USValleySEAD[valleyNum]:SetState(USValleySEAD[valleyNum], "US_AI_CAS_State", "Landed")
      setUnitStateByName(EventData.IniUnitName, "Landed")
    end
  )  

  USValleySEAD[valleyNum]:HandleEvent(EVENTS.Ejection, 
    function (group, EventData)
      D99DebugTrace("D99 in Events.Ejection")
      D99DebugTrace(EventData)
      USValleySEAD[valleyNum]:SetState(USValleySEAD[valleyNum], "US_AI_CAS_State", "Ejected")
      setUnitStateByName(EventData.IniUnitName, "Ejected")
    end
  ) 
  
  USValleySEAD[valleyNum]:HandleEvent(EVENTS.Crash, 
    function (group, EventData)
      D99DebugTrace("D99 in Events.Crash")
      D99DebugTrace(EventData)
      USValleySEAD[valleyNum]:SetState(USValleySEAD[valleyNum], "US_AI_CAS_State", "Crashed")
      setUnitStateByName(EventData.IniUnitName, "Crashed")
    end
  )  

  USValleySEAD[valleyNum]:HandleEvent(EVENTS.Dead, 
    function (group, EventData)
      D99DebugTrace("D99 in Events.Dead")
      D99DebugTrace(EventData)
      USValleySEAD[valleyNum]:SetState(USValleySEAD[valleyNum], "US_AI_CAS_State", "Dead")
      setUnitStateByName(EventData.IniUnitName, "Dead")
    end
  )  

end

function USValleySEADEngage(valleyNum)
  if (USValleySEAD[valleyNum]:IsAlive()) then
    USValleySEAD[valleyNum]:Route(USValleyRouteEngage[valleyNum])
    setUnitAndGroupState(USValleySEAD[valleyNum], "Enroute Engage")
  end
end

function USValleySEADPatrol(valleyNum)
  if (USValleySEAD[valleyNum]:IsAlive()) then
    USValleySEAD[valleyNum]:Route(USValleyRoutePatrol[valleyNum])
    setUnitAndGroupState(USValleySEAD[valleyNum], "Enroute Patrol")
  end
end

function USValleySEADRTB(valleyNum)
  if (USValleySEAD[valleyNum]:IsAlive()) then
    USValleySEAD[valleyNum]:Route(USValleyRouteRTB[valleyNum])
    setUnitAndGroupState(USValleySEAD[valleyNum], "Enroute RTB")
  end
end

function USValleySEADStatus(valleyNum)
  local USValleyStatus = USValleySEAD[valleyNum]:GetState(USValleySEAD[valleyNum], "US_AI_CAS_State") or "Unable to determine"
  local msgText = "Status: " .. USValleyStatus
  
  local tempGroupUnits = USValleySEAD[valleyNum]:GetUnits()
  if (tempGroupUnits ~= nil) then
    for key, tempUnit in pairs(tempGroupUnits) do
      local tempUnitCallSign = tempUnit:GetCallsign()
      local USValleySEADState = USValleySEADState[tempUnitCallSign]
      
      if USValleySEADState == nil then
        USValleySEADState = "Unable to determine"
      end
     
      msgText = msgText .. " - " .. tempUnitCallSign .. ": " .. USValleySEADState  
    end
  end
  MESSAGE:New(msgText, 20, "Valley " .. valleyNum):ToBlue()
end

function USValleySEADStatusAll()
  for i=1, numValleys do
    USValleySEADStatus(i)
  end
end

for i=1, numValleys do
  spawnUSValleySEAD(i)
end

-- initially I wasn't pushing the task but this is how to do such a thing. Instead changed the route above
-- to pickup from waypoint 0 and that works well enough.
--local tempTask = USAWACSPatrolGroup:EnRouteTaskAWACS()
--USAWACSPatrolGroup:PushTask(tempTask, 5)

function MenuValleyBuild(valleyNum)
  MenuBlueValley[valleyNum] = MENU_COALITION:New(coalition.side.BLUE, "Valley " .. valleyNum)
  MenuBlueValleyStatus[valleyNum] = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "Status", MenuBlueValley[valleyNum], USValleySEADStatus, valleyNum)
  MenuBlueValleyStatusAll[valleyNum] = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "Status All Flights", MenuBlueValley[valleyNum], USValleySEADStatusAll, valleyNum)
  MenuBlueValleyProceed[valleyNum] = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "SEAD Engage", MenuBlueValley[valleyNum], USValleySEADEngage, valleyNum)
  MenuBlueValleyBackToPatrol[valleyNum] = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "Back to Patrol", MenuBlueValley[valleyNum], USValleySEADPatrol, valleyNum)
  MenuBlueValleyRTB[valleyNum] = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "RTB", MenuBlueValley[valleyNum], USValleySEADRTB, valleyNum)
  MenuBlueValleyCallCAS[valleyNum] = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "Call New SEAD Flight", MenuBlueValley[valleyNum], spawnUSValleySEAD, valleyNum)
end

for i=1, numValleys do
  MenuValleyBuild(i)
end

-- Build menu for AWACS

MenuBlueAWACS = MENU_COALITION:New(coalition.side.BLUE,"AWACS")
MenuBlueAWACSCallAWACS = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "Call New AWACS Flight", MenuBlueAWACS, spawnUSAWACS)
MenuBlueAWACSRTB = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "AWACS RTB", MenuBlueAWACS, USAWACSRTB)
MenuBlueAWACSCallEscort = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "Call New Escort Flight", MenuBlueAWACS, spawnUSAWACSEscort)
MenuBlueAWACSEscortRTB = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "Escort RTB", MenuBlueAWACS, USAWACSEscortRTB)

-- setup a scheduler to set the state of the Valley flights.
BlueSEADScheduler, BlueSEADSchedulerID = SCHEDULER:New(nil,
  function ()
    for i=1, numValleys do
      if ((USValleySEAD[i] ~= nil) and (USValleySEAD[i]:IsAlive())) then
        local curState = USValleySEAD[i]:GetState(USValleySEAD[i], "US_AI_CAS_State")
        
        if (curState == "Enroute Patrol") then
          -- Check if the group is inside the patrol zone. If so then set state to Patrol On Station
          if (USValleySEAD[i]:IsPartlyInZone(ZoneUSPatrolValley[i]) or USValleySEAD[i]:IsCompletelyInZone(ZoneUSPatrolValley[i])) then
            setUnitAndGroupState(USValleySEAD[i], "Patrol On Station")
            MESSAGE:New("Patrolling on Station", 10, "Valley " .. i):ToBlue()
          end
        elseif (curState == "Enroute Engage") then
          -- Check if the group is inside the engagement zone. If so then set state to Engaging In Zone
          if (USValleySEAD[i]:IsPartlyInZone(ZoneUSEngageValley[i]) or USValleySEAD[i]:IsCompletelyInZone(ZoneUSEngageValley[i])) then
            setUnitAndGroupState(USValleySEAD[i], "Engaging In Zone")
            MESSAGE:New("Engaging in zone", 10, "Valley " .. i):ToBlue()
          end
        else
        end
      else
        USValleySEAD[i]:SetState(USValleySEAD[i], "US_AI_CAS_State", "Not Responding")
      end
    end
  end, {}, 60, 30
)

-- FIXME: Some code here below for testing and debugging purposes. Can probably delete once
-- the mission is complete.

-- This is a different method of spawning a group and copying a route from a ME place unit.
-- Wrapped in a function so that it can be spawned by F10 Radio command.
-- This was just for testing so lets remove it.
--local function Valley1SEADEngage()
--  local US_CASTaskGroup = SPAWN:New("US AI 2"):Spawn()
--  local tempRoute = GROUP:FindByName("A10 Test Group 1"):CopyRoute(1, 0, false, 0)
--
--  US_CASTaskGroup:Route(tempRoute)
--end

-- this was just for testing. Used with above function Valley1SEADEngage commenting out for now.
--MenuBlueProceed = MENU_COALITION_COMMAND:New(coalition.side.BLUE, "SEAD Engage", MenuBlue, Valley1SEADEngage)  